import rabbyt, math
from pyglet import window
import time
from pyglet import font, clock
from vec2d import*

gravity = vec2d(0,-0.02)

def get_vec_sign(vec):
    if vec[0]>0:x= 1
    else:x=-1
    if vec[1]>0:y = 1
    else:y=-1
    return x,y
    
#--------------------------------
# C O L L I S I O N  T Y P E S
#--------------------------------

def box_box_collision(obj1, obj2):
    so1 = get_vec_sign(vec2d(*obj2.xy)-vec2d(*obj1.xy))
    v_obj1 = vec2d(so1[0]*abs(obj1.SIZE[0])/2,so1[1]*abs(obj1.SIZE[1])/2)+vec2d(*obj1.xy)
    v_obj2 = vec2d(-so1[0]*obj2.W,-so1[1]*obj2.H)+vec2d(*obj2.xy)
    v = v_obj1-v_obj2
    if get_vec_sign(v) != so1:return
    if min(abs(v[0])+1,abs(v[1])) == abs(v[1]):return vec2d(0,v[1])
    else:return vec2d(v[0],0)
                                       
    
def triangle_box_collision(obj1, obj2):
    #obj1 je Trougao i statican je
    #Provera dali se dodiruje sa rectom
    axy = box_box_collision(obj1,obj2)
    if axy == None:return
    #proverava za hipotenuzu
    s = get_vec_sign(vec2d(*obj1.SIZE))
    obj2_ap_start = (vec2d(-obj2.W*s[0],-obj2.H*s[1])+vec2d(*obj2.xy)-obj1.ap_start).projection(obj1.ap)
    a_hypotenuse = obj1.ap-obj2_ap_start
    if get_vec_sign(a_hypotenuse) != s:
        return

    c = min(axy.length,a_hypotenuse.length)
    if c == axy.length:return axy
    else:return a_hypotenuse

def col_box_collision(obj1,obj2):
    #__
    #  \
    #__| obj1 je cetvrtina kruga
    axy = box_box_collision(obj1,obj2)
    if axy == None:return
    s =get_vec_sign(obj1.r)
    obj2_p = vec2d(-obj2.W*s[0],-obj2.H*s[1])+vec2d(obj2.xy)
    v = obj2_p-obj1.p

    if isinstance(obj1, CirclePart):
        if get_vec_sign(v) !=s:
            return axy
        c = obj1.p+v.normalized()*abs(obj1.r[0])-obj2_p
    else:
        v = obj2_p-(obj1.p+obj1.r)
        c = v.normalized()*abs(obj1.r[0])-v
        #print c
    if get_vec_sign(c) != s:
        return
    if min(c.length, axy.length) == axy.length:
        return axy
    else:
        return c
    
    
#-----------------------------------------
# D Y N A M I C  O B J E C T S
#-----------------------------------------

class DynamicObject(rabbyt.Sprite):
    def __init__(self, *args, **kwargs):
        rabbyt.Sprite.__init__(self, *args, **kwargs)
        self.velocity = vec2d(0,0)
        self.joint = None
    def set_joint(self,joint):
        self.joint = joint
    def update(self,dt):
        if self.joint:
            self.joint.get_object_position() 
        self.xy+= self.velocity*dt
    def handle_collision(self, obj):
        pass

class RectObject(DynamicObject):
    FRICTION = 0.03
    BOUNCE = 1
    def __init__(self, *args, **kwargs):
        DynamicObject.__init__(self, *args, **kwargs)
    def collide(self, obj):
        self.W, self.H = self.shape.width/2, self.shape.height/2
        if isinstance(obj, Box):
            c = box_box_collision(obj, self)
            if c!=None:
                self.xy+=c
                if c[0]!=0:
                    self.velocity+=self.get_friction_and_bounce(vec2d(0,obj.SIZE[1]))
                elif c[1]!=0:
                    self.velocity+=self.get_friction_and_bounce(vec2d(obj.SIZE[0],0))
        elif isinstance(obj, Triangle):
            c = triangle_box_collision(obj, self)
            if c!=None:
                self.xy+=c
                if c[0]!=0 and c[1]!=0:
                    self.velocity+=self.get_friction_and_bounce(vec2d(obj.SIZE[0],-obj.SIZE[1]))
                elif c[0] !=0:self.velocity+=self.get_friction_and_bounce(vec2d(0,obj.SIZE[1]))
                elif c[1]!=0:self.velocity+=self.get_friction_and_bounce(vec2d(obj.SIZE[0],0))
        elif isinstance(obj, CirclePart) or isinstance(obj, CirclePartConcave):
            c = col_box_collision(obj, self)
            if c!=None:
                self.xy+=c
                
                if c[0] != 0 and c[1] != 0:
                    self.velocity+=self.get_friction_and_bounce(-c.perpendicular())
                elif c[0]!=0:
                    self.velocity+=self.get_friction_and_bounce(vec2d(0,obj.SIZE[1]))
                elif c[1]!=0:
                    self.velocity+=self.get_friction_and_bounce(vec2d(obj.SIZE[0],0))
    def get_friction_and_bounce(self, vec):
        x = self.velocity.projection(vec)
        y = x-self.velocity
        x*=-self.FRICTION
        y*=self.BOUNCE
        return x+y

class Player(RectObject):
    SPEED = .1
    def __init__(self,position):
        RectObject.__init__(self, xy = position, texture = 'man.png')

        self.direction = 0
    def collide(self,obj):
        RectObject.collide(self, obj)
        
    def update(self, dt):
        if self.velocity[1]>-5:
            self.velocity += gravity
        RectObject.update(self,dt)
#---------------------------------------------------
# S T A T I C  O B J E C T S
#---------------------------------------------------



class Box(rabbyt.Sprite):
    def __init__(self, pos, SIZE = (100,100)):

        self.SIZE = SIZE
        w = self.SIZE[0]/2
        h = self.SIZE[1]/2
        rabbyt.Sprite.__init__(self,xy= pos, shape =(-w,h,w,-h), alpha =0.7)
        
class Triangle(rabbyt.Sprite):
    hypotenuse_h = None
    def __init__(self,pos,SIZE = (100,100)):
        #d predstavlja kako je troguao namesten
        self.SIZE =SIZE
        w,h = self.SIZE[0]/2,self.SIZE[1]/2
        if Triangle.hypotenuse_h == None:Triangle.hypotenuse_h = abs(self.SIZE[0]*self.SIZE[1]/math.hypot(self.SIZE[0],self.SIZE[1]))
        rabbyt.Sprite.__init__(self,xy= pos,texture = 'triangle.png', shape =(-w,h,w,-h), alpha =0.7)
        self.ap = vec2d(*self.SIZE).perpendicular_normal()*Triangle.hypotenuse_h
        s = get_vec_sign(vec2d(*self.SIZE))
        self.ap = vec2d(abs(self.ap[0])*s[0],abs(self.ap[1])*s[1])
        self.ap_start = vec2d(self.x - w, self.y - h)

class CirclePart(rabbyt.Sprite):
    def __init__(self,pos, SIZE= (100,100)):
        w,h = SIZE[0]/2,SIZE[1]/2
        self.SIZE = SIZE
        rabbyt.Sprite.__init__(self, xy = pos, texture = 'circlepart.png', shape = (-w,h,w,-h),alpha =0.7)
        self.r = vec2d(SIZE[0],SIZE[1])
        self.p = vec2d(self.x-w,self.y-h)
        
class CirclePartConcave(rabbyt.Sprite):
    def __init__(self,pos, SIZE= (100,100)):
        w,h = SIZE[0]/2,SIZE[1]/2
        self.SIZE = SIZE
        rabbyt.Sprite.__init__(self, xy = pos, texture = 'circlepartconcave.png', shape = (-w,h,w,-h),alpha =0.7)
        self.r = vec2d(SIZE[0],SIZE[1])
        self.p = vec2d(self.x-w,self.y-h)        

        
#----------------------------
# T E S T I N G
#----------------------------
if __name__ == '__main__':
    clock.schedule(rabbyt.add_time)
    w, h = 640,480
    win = window.Window(width =w, height = h, vsync = False)
    keystate = window.key.KeyStateHandler()
    win.push_handlers(keystate)
    rabbyt.set_default_attribs()

    d = Player((0,200))
    b = CirclePartConcave((85,0),SIZE = (70,70))
    b2 = Box((0,0),SIZE = (100,70))
    b3 = Box((-100,0),SIZE = (100,70))

    while not win.has_exit:
        dt = clock.tick()*100
        win.dispatch_events()
        #Handlers
        if keystate[window.key.UP] and d.velocity[1]<3:
            d.velocity[1]+=.06
        if keystate[window.key.DOWN] and d.velocity[1]>-3:
            d.velocity[1]-=.06   
        if keystate[window.key.RIGHT] and d.velocity[0] < 5:
            d.velocity[0] +=.04
        if keystate[window.key.LEFT] and d.velocity[0] > -5:
            d.velocity[0]-= .04

        if keystate[window.key.SPACE]:
            
            d.velocity = vec2d(0,0)
    
        #Physics
        d.update(dt)
        d.collide(b)
        d.collide(b2)
        d.collide(b3)
        #Graphycs
        rabbyt.set_viewport((640,480),(-w/2,h/2,w/2,-h/2))
        rabbyt.clear((0.5,0.5,0.8))
        b.render()
        b2.render()
        b3.render()
        d.render()

        win.flip()
    win.close()
